import filecmp, os, platform, shutil, time, traceback
from fpcommon import is_mac, is_windows
from os.path import *

class BackupCommand:

	def __init__( self, inp, out, path ):
		self.input = inp
		self.output = out
		self.rsrc_path = path
		self.backup_path = None

	def error( self, msg ):
		return { 'status' : "error", 'message' : msg }

	def save_path( self ):
		return join( self.rsrc_path, self.save_name )

	def run_command( self, args ):
		try:
			if args[0] == 'start':
				result = self.start_directory()
			elif args[0] == 'copy':
				result = self.copy_file( args[1] )
			elif args[0] == 'verify':
				result = self.verify_file( args[1], args[2] )
			elif args[0] == 'abort':
				result = self.abort_copy()
			elif args[0] == 'delete':
				result = self.delete_file( args[1] )
			else:
				if len( args ) == 0: args = ["<unknown>"]
				return self.error( "unrecognized picasa command: %s" % args[0] )
		except:
			result = { 'message' : "exception in command '%s'" % args[0],
						'status' : "error", 'traceback' : traceback.format_exc() }
		return result

	def start_directory( self ):
		result = { 'status' : "error", 'message' : "could not create backup directory" }
		userdir = None
		if is_windows():
			userdir = self.win_pictures_dir( userdir )
		elif is_mac():
			userdir = expanduser( "~" )
			userdir = self.mac_pictures_dir( userdir )
		if userdir and isdir( userdir ):
			# <localize>
			dirname = time.strftime( "Flip-Pal %Y-%b-%d, %I.%M.%S %p", time.localtime() )
			# </localize>
			self.backup_path = join( userdir, dirname )
			os.mkdir( self.backup_path  )
			if isdir(  self.backup_path ):
				result = { 'status' : "ok", 'directory' : self.backup_path }
		return result

	def win_pictures_dir( self, userdir ):
		if not userdir or not isdir( userdir ):
			try:
				userdir = os.environ["USERPROFILE"]
			except:
				pass
		# Now, watch carefully...
		if userdir and isdir( userdir ):
			# <localize>
			tmp = join( userdir, "Pictures" ) # Win7
			# </localize>
			if isdir( tmp ):
				userdir = tmp
			else:
				# Documents at same level as Pictures in 7
				# <localize>
				tmp = join( userdir, "Documents" ) # Win7 fallback
				# </localize>
				if isdir( tmp ):
					userdir = tmp
				else:
					# My Documents\My Pictures on XP
					# <localize>
					tmp = join( userdir, "My Documents" ) # WinXP fallback
					# </localize>
					if isdir( tmp ):
						userdir = tmp
						# <localize>
						tmp = join( userdir, "My Pictures" ) # Win XP
						# </localize>
						if isdir( tmp ):
							userdir = tmp
		return userdir

	def mac_pictures_dir( self, userdir ):
		if not userdir or not isdir( userdir ):
			try:
				userdir = os.environ["HOME"]
			except:
				pass
		if userdir and isdir( userdir ):
			# <localize>
			tmp = join( userdir, "Pictures" )
			# </localize>
			if isdir( tmp ):
				userdir = tmp
			else:
				# <localize>
				tmp = join( userdir, "Documents" )
				# </localize>
				if isdir( tmp ):
					userdir = tmp
		return userdir

	def copy_file( self, path ):
		if not self.backup_path or not isdir( self.backup_path ):
			return { 'status' : "error", 'message' : "need successful 'start' first" }
		name = basename( path )
		dest = join( self.backup_path, name )
		try:
			shutil.copyfile( path, dest )
		except:
			return { 'status' : "error", 'traceback' : traceback.format_exc(),
					'message' : "exception in copy command for '%s'" % path }
		if isfile( dest ):
			try:
				shutil.copystat( path, dest )
			except:
				pass
		return { 'status' : "ok", 'result' : dest }

	def verify_file( self, path1, path2 ):
		if not path1 or not isfile( path1 ):
			return { 'status' : "error", 'message' : "does not exist: '%s'" % path1 }
		if not path2 or not isfile( path2 ):
			return { 'status' : "error", 'message' : "does not exist: '%s'" % path2 }
		try:
			if filecmp.cmp( path1, path2 ):
				return { 'status' : "ok" }
			else:
				return { 'status' : "error", 'message' : "files did not verify" }
		except:
			return { 'status' : "error", 'traceback' : traceback.format_exc(),
					'message' : "exception verifying files" }

	def abort_copy( self ):
		if not isdir( self.backup_path ):
			return { 'status' : "error", 'message' : "no directory: %s" % self.backup_path }
		# clean up incomplete directory
		for f in os.listdir( self.backup_path ):
			try:
				os.remove( join( self.backup_path, f ))
			except:
				pass
		if len( os.listdir( self.backup_path )) == 0:
			try:
				os.rmdir( self.backup_path )
			except:
				pass
			if not isdir( self.backup_path ):
				return { 'status' : "ok" }
		return { 'status' : "error", 'message' : "could not remove '%s'" % self.backup_path }

	# Generic function, does not depend on BackupCommand state

	def delete_file( self, path ):
		if not isfile( path ):
			return { 'status' : "error", 'message' : "invalid file: '%s'" % path }
		try:
			os.remove( path )
			[pdir, name] = split( path )
			thumb = join( pdir, ".thumb", name )
			if isfile( thumb ):
				os.remove( thumb )
		except:
			pass
		if isfile( path ):
			return { 'status' : "error", 'message' : "tried and failed to remove: '%s'" % path }
		else:
			return { 'status' : "ok" }

